#!/usr/bin/python

import os
import unittest
import logging

from confini import Config

logging.basicConfig(level=logging.DEBUG)
logg = logging.getLogger()

class TestEnv(unittest.TestCase):

    wd = os.path.dirname(__file__)

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_dict_override(self):
        inidir = os.path.join(self.wd, 'files')
        c = Config(inidir)
        c.process()

        override_dict = {
                'FOO_BAR': '666',
                'XYZZY_BERT': 'oscar',
                'BAR_FOO': None,
                }
        c.dict_override(override_dict, 'arbitrary dict')

        expect = {
            'FOO_BAR': '666',
            'FOO_BAZ': '029a',
            'BAR_FOO': 'oof',
            'XYZZY_BERT': 'oscar',
                }
        self.assertDictEqual(expect, c.store)

        override_dict = {
                'BAR_FOO': 'barbarbar',
                }
        c.dict_override(override_dict, 'arbitrary dict')

        expect = {
            'FOO_BAR': '666',
            'FOO_BAZ': '029a',
            'BAR_FOO': 'barbarbar',
            'XYZZY_BERT': 'oscar',
                }
        self.assertDictEqual(expect, c.store)



    def test_env_a_override(self):
        os.environ['FOO_BAR'] = '43'
        inidir = os.path.join(self.wd, 'files')
        c = Config(inidir)
        c.process()

        os.environ['ZZZ_FOO_BAR'] = '44'
        inidir = os.path.join(self.wd, 'files')
        c = Config(inidir, 'ZZZ')
        c.process()
        expect = {
            'FOO_BAR': '44',
            'FOO_BAZ': '029a',
            'BAR_FOO': 'oof',
            'XYZZY_BERT': 'ernie',
                }
        self.assertDictEqual(expect, c.store)

        os.environ['ZZZ_FOO_BAR'] = ''
        inidir = os.path.join(self.wd, 'files')
        c = Config(inidir, 'ZZZ')
        c.process()
        expect = {
            'FOO_BAR': '42',
            'FOO_BAZ': '029a',
            'BAR_FOO': 'oof',
            'XYZZY_BERT': 'ernie',
                }
        self.assertDictEqual(expect, c.store)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python

import os
import unittest
import logging

from confini import Config

logging.basicConfig(level=logging.DEBUG)
logg = logging.getLogger()

class TestBasic(unittest.TestCase):

    wd = os.path.dirname(__file__)

    def setUp(self):
        pass


    def tearDown(self):
        pass


    def test_parse_default(self):
        inidir = os.path.join(self.wd, 'files/default')
        c = Config(inidir)
        c.process()
        r = c.get('FOO_BAR', 'plugh')
        self.assertEqual(r, 'xyzzy')
        r = c.get('FOO_BAZ', 'plugh')
        self.assertEqual(r, 'plugh')
        r = c.get('FOO_BAZ')
        self.assertEqual(r, None)
        with self.assertRaises(KeyError):
            r = c.get('FOO_BAZBAZ')


    def test_overwrite_guard(self):
        inidir = os.path.join(self.wd, 'files/default')
        c = Config(inidir)
        c.process()
        with self.assertRaises(AttributeError):
            c.add('xxx', 'FOO_BAR')
        c.add('xxx', 'FOO_BAR', exists_ok=True)
        r = c.get('FOO_BAR')
        self.assertEqual(r, 'xxx')


    def test_parse_two_files(self):
        inidir = os.path.join(self.wd, 'files')
        c = Config(inidir)
        c.process()
        c.require('BERT', 'XYZZY')
        expect = {
            'FOO_BAR': '42',
            'FOO_BAZ': '029a',
            'BAR_FOO': 'oof',
            'XYZZY_BERT': 'ernie',
                }
        self.assertDictEqual(expect, c.store)


    def test_require(self):
        inidir = os.path.join(self.wd, 'files')
        c = Config(inidir)
        c.require('BERT', 'XYZZY')
        self.assertTrue(c.validate())
        c.require('ERNIE', 'XYZZY')
        self.assertFalse(c.validate())
        logg.debug(c)


    def test_all(self):
        inidir = os.path.join(self.wd, 'files')
        c = Config(inidir)
        a = c.all()
        self.assertEqual(a, c.store)


if __name__ == '__main__':
    unittest.main()



# standard imports
import sys
import logging
import argparse

# local imports
from confini import Config

logging.basicConfig(level=logging.WARNING)
logg = logging.getLogger()


argparser = argparse.ArgumentParser()
argparser.add_argument('-z', action='store_true', help='Truncate values in output')
argparser.add_argument('-v', action='store_true', help='Be verbose')
argparser.add_argument('config_dir', type=str, help='Configuation directory to parse')
args = argparser.parse_args()

if args.v:
    logg.setLevel(logging.DEBUG)


def main():
    c = Config(args.config_dir)
    c.process()
    for k in c.store.keys():
        v = c.get(k)
        if args.z or v == None:
            v = ''
        print('{}={}'.format(k, v))


if __name__ == "__main__":
    main()



#!/usr/bin/python

import os
import unittest
import logging

from confini import Config

logging.basicConfig(level=logging.DEBUG)
logg = logging.getLogger()


class TestTranslate(unittest.TestCase):

    wd = os.path.dirname(__file__)

    def setUp(self):
        pass


    def tearDown(self):
        pass


    def test_parse_default(self):
        inidir = os.path.join(self.wd, 'files/translate')
        c = Config(inidir)
        c.process()

        self.assertTrue(c.get('TRUE_A'))
        self.assertTrue(c.get('TRUE_B'))
        self.assertTrue(c.get('TRUE_C'))

        self.assertFalse(c.true('FALSE_A'))
        self.assertFalse(c.true('FALSE_B'))
        self.assertFalse(c.true('FALSE_C'))
        self.assertIsNone(c.get('FALSE_D'))

        o = {
            'TRUE_A': True,
            'FALSE_A': False,
                }
        c.dict_override(o, 'test')
        self.assertTrue(c.true('TRUE_A'))
        self.assertFalse(c.true('FALSE_A'))

     
if __name__ == '__main__':
    unittest.main()



from setuptools import setup

f = open('./README.md', 'r')
long_description = f.read()
f.close()

setup(
        name='confini',
        version='0.3.6rc2',
        description='Parse, verify and merge all ini files in a single directory',
        author='Louis Holbrook',
        author_email='dev@holbrook.no',
        license='GPL3',
        long_description=long_description,
        long_description_content_type='text/markdown',
        install_requires=[
            'python-gnupg>=0.4.6,<0.5.0',
        ],
        packages=[
            'confini',
            'confini.runnable',
        ],
        url='https://gitlab.com/nolash/python-confini',
        )



import logging
import sys
import os
import tempfile
import configparser
import re
import gnupg

from .error import DecryptError

logg = logging.getLogger('confini')

current_config = None

gpg = gnupg.GPG(
    verbose=False,
    use_agent=True,
        )
gpg.encoding = 'utf-8'


def set_current(conf, description=''):
    global current_config
    logg.debug('setting current config ({})'.format(description))
    current_config = conf 


class Config:

    parser = configparser.ConfigParser(strict=True)

    def __init__(self, config_dir, env_prefix=None, decrypt=True):
        if not os.path.isdir(config_dir):
            raise OSError('{} is not a directory'.format(config_dir))
        self.dir = os.path.realpath(config_dir)
        self.required = {}
        self.censored = {}
        self.store = {}
        self.decrypt = decrypt
        self.env_prefix = None
        if env_prefix != None:
            logg.info('using prefix {} for environment variable override matches'.format(env_prefix))
            self.env_prefix = '{}_'.format(env_prefix)


    def add(self, value, constant_name, exists_ok=False):
        if self.store.get(constant_name) != None:
            if not exists_ok:
                raise AttributeError('config key {} already exists'.format(constant_name))
            else:
                logg.debug('overwriting key {}'.format(constant_name))
        self.store[constant_name] = value


    def censor(self, identifier, section=None):
        constant_name = ''
        if section != None:
            constant_name = Config.to_constant_name(identifier, section)
        else:
            constant_name = identifier
        self.censored[constant_name] = True


    def require(self, directive, section):
        if self.required.get(section) == None:
            self.required[section] = []
        self.required[section].append(directive)


    def validate(self):
        for k in self.required.keys():
            for v in self.required[k]:
                try:
                    _ = self.parser[k][v]
                except:
                    return False
        return True


    @staticmethod
    def to_constant_name(directive, section):
        return '{}_{}'.format(section.upper(), directive.upper())


    def _sections_override(self, dct, dct_description):
        for s in self.parser.sections():
            for k in self.parser[s]:
                cn = Config.to_constant_name(k, s)
                self.override(cn, self.parser[s][k], dct, dct_description)


    def dict_override(self, dct, dct_description):
        for k in dct.keys():
            try:
                self.override(k, self.store[k], dct, dct_description)
            except KeyError:
                logg.warning('override key {} have no match in config store'.format(k))


    def override(self, cn, v, dct, dct_description):
        cn_env = cn
        if self.env_prefix != None:
            cn_env = self.env_prefix + cn
        val = dct.get(cn_env)
        if val == None or val == '':
            val = self.store.get(cn, v)
        else:
            logg.info('{} {} overrides {}'.format(dct_description, cn_env, cn))
        self.add(val, cn, exists_ok=True)


    def process(self, set_as_current=False):
        """Concatenates all .ini files in the config directory attribute and parses them to memory
        """
        tmp = tempfile.NamedTemporaryFile(delete=False)
        tmpname = tmp.name
        for filename in os.listdir(self.dir):
            if re.match(r'.+\.ini$', filename) == None:
                logg.debug('skipping file {}'.format(filename))
                continue
            logg.info('reading file {}'.format(filename))
            f = open(os.path.join(self.dir, filename), 'rb')
            while 1:
                data = f.read()
                if not data:
                    break
                tmp.write(data)
            f.close()
        tmp.close()
        self.parser.read(tmpname)
        os.unlink(tmpname)
        self._sections_override(os.environ, 'environment variable')
        if set_as_current:
            set_current(self, description=self.dir)



    def _decrypt(self, k, v):
        if type(v).__name__ != 'str':
            logg.debug('entry {} is not type str'.format(k))
            return v
        if self.decrypt:
            m = re.match(r'^\!gpg\((.*)\)', v)
            if m != None:
                filename = m.group(1)
                if filename[0] != '/':
                    filename = os.path.join(self.dir, filename)
                f = open(filename, 'rb')
                logg.debug('decrypting entry {} in file {}'.format(k, f))
                d = gpg.decrypt_file(f)
                if not d.ok:
                    raise DecryptError()
                v = str(d)
                f.close()
        return v


    def get(self, k, default=None):
        v = self.store[k]
        if v == None:
            if default != None:
                logg.debug('returning default value for empty value {}'.format(k))
            return default
        if type(v).__name__ == 'str' and v == '':
            if default != None:
                logg.debug('returning default value for empty string value {}'.format(k))
                return default
            else:
                return None

        return self._decrypt(k, v)


    def all(self):
        return self.store


    def true(self, k):
        v = self.store.get(k)
        if type(v).__name__ == 'bool':
            logg.debug('entry {} is already bool'.format(k))
            return v
        d = self._decrypt(k, v)
        if d.lower() not in ['true', 'false', '0', '1', 'on', 'off']:
            raise ValueError('{} not a boolean value'.format(k))
        return d.lower() in ['true', '1', 'on']


    def __str__(self):
        ls = []
        for k in self.store.keys():
            v = ''
            try:
                _ = self.censored[k]
                v = '***'
            except:
                v = self.store[k]

            ls.append('{}={}'.format(k, v))

        return '\n'.join(ls)


    def __repr__(self):
        return "<Config '{}'>".format(self.dir)




def config_from_environment():
    config_dir = config_dir_from_environment()
    c = Config(config_dir)
    c.process()
    return c


def config_dir_from_environment():
    return os.environ.get('CONFINI_DIR')
